package com.usoft.sso.rest.domain;

import com.sso.rest.utils.exception.SSORuntimeException;
import com.usoft.protobuf.utils.ProtoBufUtil;
import com.usoft.sso.grpc.api.entity.Enterprise;
import com.usoft.sso.grpc.api.entity.UserInfo;
import com.usoft.sso.grpc.api.protobuf.*;
import com.google.protobuf.InvalidProtocolBufferException;
import com.usoft.sso.rest.api.protobuf.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author: huyy
 * @date: 2018-08-22 14:57
 **/
@Service
public class SSODomain {

    @Autowired
    private SSOGrpcServiceGrpc.SSOGrpcServiceBlockingStub ssoUserSBS;

    /**
     * 账户中心保存用户信息接口
     *
     * @param req
     * @return
     * @throws InvalidProtocolBufferException
     */
    public SaveUserInfoResp.Builder save(SaveUserInfoReq req) throws InvalidProtocolBufferException {
        // 创建grpc request
        com.usoft.sso.grpc.api.entity.UserInfo.Builder userInfo = ProtoBufUtil.toProtoBuf(com.usoft.sso.grpc.api.entity.UserInfo.newBuilder(), req.getUserInfo());
        SaveUserInfoRequest.Builder builder = SaveUserInfoRequest.newBuilder();
        builder.setUserInfo(userInfo);
        SaveUserInfoResponse response = ssoUserSBS.save(builder.build());
        if (response.getResponseHeader().getCode() != 0) {
            throw new SSORuntimeException(response.getResponseHeader().getCode(), response.getResponseHeader().getMsg());
        }
        return SaveUserInfoResp.newBuilder();
    }

    /**
     * 账户中心用户 绑定/解绑
     *
     * @param req
     * @return
     * @throws InvalidProtocolBufferException
     */
    public BindUserInfoResp.Builder bindUser(BindUserInfoReq req) throws InvalidProtocolBufferException {
        //实例化BindUserInfoRequest
        BindUserInfoRequest.Builder builder = BindUserInfoRequest.newBuilder();
        //出入请求参数
        builder.setUserUU(req.getUserUU());
        builder.setSpaceUU(req.getSpaceUU());
        builder.setType(req.getType());
        BindUserInfoResponse response = ssoUserSBS.bindUser(builder.build());
        if (response.getResponseHeader().getCode() != 0) {
            throw new SSORuntimeException(response.getResponseHeader().getCode(), response.getResponseHeader().getMsg());
        }
        return BindUserInfoResp.newBuilder();
    }

    /**
     * 企业信息保存+更换管理员同步信息接口
     *
     * @param req
     * @return
     * @throws InvalidProtocolBufferException
     */
    public UpdateEntInfoResp.Builder update(UpdateEntInfoReq req) throws InvalidProtocolBufferException {
        // 创建grpc request
        UserInfo.Builder userInfo = ProtoBufUtil.toProtoBuf(UserInfo.newBuilder(), req.getUserInfo());
        Enterprise.Builder enterpriseInfo = ProtoBufUtil.toProtoBuf(Enterprise.newBuilder(), req.getEnterpriseInfo());
        UpdateEntInfoRequest.Builder builder = UpdateEntInfoRequest.newBuilder();
        builder.setUser(userInfo);
        builder.setEnterprise(enterpriseInfo);
        UpdateEntInfoResponse response = ssoUserSBS.update(builder.build());
        if (response.getResponseHeader().getCode() != 0) {
            throw new SSORuntimeException(response.getResponseHeader().getCode(), response.getResponseHeader().getMsg());
        }
        return UpdateEntInfoResp.newBuilder();
    }


}
