package com.usoft.sso.rest.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.protobuf.InvalidProtocolBufferException;
import com.sso.rest.utils.common.RespUtil;
import com.sso.rest.utils.common.TypeChangeUtil;
import com.sso.rest.utils.exception.SSORuntimeException;
import com.usoft.protobuf.utils.ProtoBufUtil;
import com.usoft.sso.rest.api.entity.EnterpriseInfo;
import com.usoft.sso.rest.api.entity.RespHeader;
import com.usoft.sso.rest.api.entity.UserInfo;
import com.usoft.sso.rest.api.protobuf.UpdateEntInfoReq;
import com.usoft.sso.rest.api.protobuf.UpdateEntInfoResp;
import com.usoft.sso.rest.domain.SSODomain;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;

/**
 * @author: huyy
 * @date: 2018/8/23 15:11
 */
@RestController
public class EnterpriseController {
    @Resource
    private SSODomain ssoDomain;
    private static final Logger LOGGER = LoggerFactory.getLogger(EnterpriseController.class);

    /**
     * 企业信息保存+更换管理员同步信息接口
     *
     * @param json
     * @return
     * @throws UnsupportedEncodingException
     * @throws InvalidProtocolBufferException
     */
    @PostMapping(value = "/api/public/enterprise")
    public String updateEntInfo(@RequestBody String json) throws UnsupportedEncodingException, InvalidProtocolBufferException {
        //账户中心传过来的数据比较特别，需要特殊解码
        JSONObject object = TypeChangeUtil.stringToJSONObj(json);
        EnterpriseInfo.Builder enBuilder = ProtoBufUtil.toProtoBuf(EnterpriseInfo.newBuilder(), object.toJSONString());
        UserInfo.Builder userBuilder = ProtoBufUtil.toProtoBuf(UserInfo.newBuilder(), object.toJSONString());
        UpdateEntInfoResp.Builder resp = UpdateEntInfoResp.newBuilder();
        UpdateEntInfoReq.Builder req = UpdateEntInfoReq.newBuilder();
        req.setEnterpriseInfo(enBuilder);
        req.setUserInfo(userBuilder);
        try {
            resp = ssoDomain.update(req.build());
            resp.setRespHeader(RespHeader.newBuilder());
            LOGGER.info("更新企业信息[UserController.updateEntInfo]正常，参数：{}", ProtoBufUtil.toJSONHasTryCatch(req.build()));
        } catch (SSORuntimeException e) {
            LOGGER.warn("更新企业信息[UserController.updateEntInfo]告警，参数：{}", json, e);
            RespHeader respHeader = RespUtil.getRespHeader(e.getErrorCode(), e.getErrorMessage());
            resp.setRespHeader(respHeader);
        } catch (Exception e) {
            LOGGER.error("更新企业信息[UserController.updateEntInfo]异常，参数：{}", json, e);
            RespHeader respHeader = RespUtil.getRespHeader(SSORuntimeException.ERROR_CODE_100, e.getMessage());
            resp.setRespHeader(respHeader);
        }
        return ProtoBufUtil.toJSONHasTryCatch(resp.build());
    }

}
